﻿using System.Linq.Expressions;
using System.Reflection;

namespace TestProject;

/// <summary>
/// 속성 헬퍼
/// </summary>
/// <typeparam name="TSource">소스 타입</typeparam>
public static class PropertyHelper<TSource>
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 속성 정보 구하기 - GetPropertyInfo<TProperty>(selector)

    /// <summary>
    /// 속성 정보 구하기
    /// </summary>
    /// <typeparam name="TProperty">속성 타입</typeparam>
    /// <param name="selector">셀렉터</param>
    /// <returns>속성 정보</returns>
    public static PropertyInfo GetPropertyInfo<TProperty>(Expression<Func<TSource, TProperty>> selector)
    {
        Expression body = selector;

        if(body is LambdaExpression)
        {
            body = ((LambdaExpression)body).Body;
        }

        switch(body.NodeType)
        {
            case ExpressionType.MemberAccess : return (PropertyInfo)((MemberExpression)body).Member;
            default                          : throw new InvalidOperationException();
        }
    }

    #endregion
}